@@ -23,6 +23,7 @@
 
 #include <sys/ioctl.h>
 #include <sys/socket.h>
+#include <sys/un.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <netinet/tcp.h>
@@ -39,6 +40,8 @@
 #include <errno.h>
 #include <getopt.h>
 #include <stdarg.h>
+#include <stdbool.h>
+#include <time.h>
 
 #include <linux/ioctl.h>
 #define MY_NAME "nbd_client"
@@ -72,8 +75,11 @@ int check_conn(char* devname, int do_print) {
 		}
 	}
 	len=read(fd, buf, 256);
-	if(len >= 0)
-		buf[len]='\0';
+	if(len < 0) {
+		perror("could not read from server");
+		return 2;
+	}
+	buf[(len < 256) ? len : 255]='\0';
 	if(do_print) printf("%s\n", buf);
 	return 0;
 }
@@ -124,21 +130,49 @@ int opennet(char *name, char* portstr, int sdp) {
 
 	if (rp == NULL) {
 		err_nonfatal("Socket failed: %m");
-		return -1;
+		sock = -1;
+		goto err;
 	}
 
 	setmysockopt(sock);
-
+err:
 	freeaddrinfo(ai);
 	return sock;
 }
 
+int openunix(const char *path) {
+	int sock;
+	struct sockaddr_un un_addr;
+	memset(&un_addr, 0, sizeof(un_addr));
+
+	un_addr.sun_family = AF_UNIX;
+	if (strnlen(path, sizeof(un_addr.sun_path)) == sizeof(un_addr.sun_path)) {
+		err_nonfatal("UNIX socket path too long");
+		return -1;
+	}
+
+	strncpy(un_addr.sun_path, path, sizeof(un_addr.sun_path) - 1);
+
+	if ((sock = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
+		err_nonfatal("SOCKET failed");
+		return -1;
+	};
+
+	if (connect(sock, &un_addr, sizeof(un_addr)) == -1) {
+		err_nonfatal("CONNECT failed");
+		close(sock);
+		return -1;
+	}
+	return sock;
+}
+
 void ask_list(int sock) {
 	uint32_t opt;
 	uint32_t opt_server;
 	uint32_t len;
 	uint32_t reptype;
 	uint64_t magic;
+	int rlen;
 	const int BUF_SIZE = 1024;
 	char buf[BUF_SIZE];
 
@@ -188,11 +222,12 @@ void ask_list(int sock) {
 					break;
 			}
 			if(len > 0 && len < BUF_SIZE) {
-				if(len=read(sock, buf, len) < 0) {
+				if((rlen=read(sock, buf, len)) < 0) {
 					fprintf(stderr, "\nE: could not read error message from server\n");
+				} else {
+					buf[rlen] = '\0';
+					fprintf(stderr, "Server said: %s\n", buf);
 				}
-				buf[len] = '\0';
-				fprintf(stderr, "Server said: %s\n", buf);
 			}
 			exit(EXIT_FAILURE);
 		} else {
@@ -229,82 +264,65 @@ void ask_list(int sock) {
 		err("Failed writing length");
 }
 
-void negotiate(int sock, u64 *rsize64, u32 *flags, char* name, uint32_t needed_flags, uint32_t client_flags, uint32_t do_opts) {
+void negotiate(int sock, u64 *rsize64, uint16_t *flags, char* name, uint32_t needed_flags, uint32_t client_flags, uint32_t do_opts) {
 	u64 magic, size64;
 	uint16_t tmp;
+	uint16_t global_flags;
 	char buf[256] = "\0\0\0\0\0\0\0\0\0";
+	uint32_t opt;
+	uint32_t namesize;
 
 	printf("Negotiation: ");
-	if (read(sock, buf, 8) < 0)
-		err("Failed/1: %m");
-	if (strlen(buf)==0)
-		err("Server closed connection");
+	readit(sock, buf, 8);
 	if (strcmp(buf, INIT_PASSWD))
 		err("INIT_PASSWD bad");
 	printf(".");
-	if (read(sock, &magic, sizeof(magic)) < 0)
-		err("Failed/2: %m");
+	readit(sock, &magic, sizeof(magic));
 	magic = ntohll(magic);
-	if(name) {
-		uint32_t opt;
-		uint32_t namesize;
-
-		if (magic != opts_magic) {
-			if(magic == cliserv_magic) {
-				err("It looks like you're trying to connect to an oldstyle server with a named export. This won't work.");
-			}
+	if (magic != opts_magic) {
+		if(magic == cliserv_magic) {
+			err("It looks like you're trying to connect to an oldstyle server. This is no longer supported since nbd 3.10.");
 		}
-		printf(".");
-		if(read(sock, &tmp, sizeof(uint16_t)) < 0) {
-			err("Failed reading flags: %m");
-		}
-		*flags = ((u32)ntohs(tmp));
-		if((needed_flags & *flags) != needed_flags) {
-			/* There's currently really only one reason why this
-			 * check could possibly fail, but we may need to change
-			 * this error message in the future... */
-			fprintf(stderr, "\nE: Server does not support listing exports\n");
-			exit(EXIT_FAILURE);
-		}
-
-		client_flags = htonl(client_flags);
-		if (write(sock, &client_flags, sizeof(client_flags)) < 0)
-			err("Failed/2.1: %m");
-
-		if(do_opts & NBDC_DO_LIST) {
-			ask_list(sock);
-			exit(EXIT_SUCCESS);
-		}
-
-		/* Write the export name that we're after */
-		magic = htonll(opts_magic);
-		if (write(sock, &magic, sizeof(magic)) < 0)
-			err("Failed/2.2: %m");
-
-		opt = ntohl(NBD_OPT_EXPORT_NAME);
-		if (write(sock, &opt, sizeof(opt)) < 0)
-			err("Failed/2.3: %m");
-		namesize = (u32)strlen(name);
-		namesize = ntohl(namesize);
-		if (write(sock, &namesize, sizeof(namesize)) < 0)
-			err("Failed/2.4: %m");
-		if (write(sock, name, strlen(name)) < 0)
-			err("Failed/2.4: %m");
-	} else {
-		if (magic != cliserv_magic) {
-			if(magic != opts_magic)
-				err("Not enough cliserv_magic");
-			else
-				err("It looks like you're trying to connect to a newstyle server with the oldstyle protocol. Try the -N option.");
-		}
-		printf(".");
+	}
+	printf(".");
+	readit(sock, &tmp, sizeof(uint16_t));
+	global_flags = ntohs(tmp);
+	if((needed_flags & global_flags) != needed_flags) {
+		/* There's currently really only one reason why this
+		 * check could possibly fail, but we may need to change
+		 * this error message in the future... */
+		fprintf(stderr, "\nE: Server does not support listing exports\n");
+		exit(EXIT_FAILURE);
 	}
 
-	if (read(sock, &size64, sizeof(size64)) <= 0) {
-		if (!errno)
-			err("Server closed connection");
-		err("Failed/3: %m\n");
+	if (global_flags & NBD_FLAG_NO_ZEROES) {
+		client_flags |= NBD_FLAG_C_NO_ZEROES;
 	}
+	client_flags = htonl(client_flags);
+	if (write(sock, &client_flags, sizeof(client_flags)) < 0)
+		err("Failed/2.1: %m");
+
+	if(do_opts & NBDC_DO_LIST) {
+		ask_list(sock);
+		exit(EXIT_SUCCESS);
+	}
+
+	/* Write the export name that we're after */
+	magic = htonll(opts_magic);
+	if (write(sock, &magic, sizeof(magic)) < 0)
+		err("Failed/2.2: %m");
+
+	opt = ntohl(NBD_OPT_EXPORT_NAME);
+	if (write(sock, &opt, sizeof(opt)) < 0)
+		err("Failed/2.3: %m");
+	namesize = (u32)strlen(name);
+	namesize = ntohl(namesize);
+	if (write(sock, &namesize, sizeof(namesize)) < 0)
+		err("Failed/2.4: %m");
+	if (write(sock, name, strlen(name)) < 0)
+		err("Failed/2.4: %m");
+
+	readit(sock, &size64, sizeof(size64));
 	size64 = ntohll(size64);
 
 	if ((size64>>12) > (uint64_t)~0UL) {
@@ -313,23 +331,127 @@ void negotiate(int sock, u64 *rsize64, u32 *flags, char* name, uint32_t needed_f
 	} else
 		printf("size = %luMB", (unsigned long)(size64>>20));
 
-	if(!name) {
-		if (read(sock, flags, sizeof(*flags)) < 0)
-			err("Failed/4: %m\n");
-		*flags = ntohl(*flags);
-	} else {
-		if(read(sock, &tmp, sizeof(tmp)) < 0)
-			err("Failed/4: %m\n");
-		*flags |= (uint32_t)ntohs(tmp);
-	}
+	readit(sock, &tmp, sizeof(tmp));
+	*flags = (uint32_t)ntohs(tmp);
 
-	if (read(sock, &buf, 124) < 0)
-		err("Failed/5: %m\n");
+	if (!(global_flags & NBD_FLAG_NO_ZEROES)) {
+		readit(sock, &buf, 124);
+	}
 	printf("\n");
 
 	*rsize64 = size64;
 }
 
+bool get_from_config(char* cfgname, char** name_ptr, char** dev_ptr, char** hostn_ptr, int* bs, int* timeout, int* persist, int* swap, int* sdp, int* b_unix, char**port) {
+	int fd = open(SYSCONFDIR "/nbdtab", O_RDONLY);
+	bool retval = false;
+	if(fd < 0) {
+		fprintf(stderr, "while opening %s: ", SYSCONFDIR "/nbdtab");
+		perror("could not open config file");
+		goto out;
+	}
+	off_t size = lseek(fd, 0, SEEK_END);
+	lseek(fd, 0, SEEK_SET);
+	void *data = NULL;
+	char *fsep = "\n\t# ";
+	char *lsep = "\n#";
+
+	if(size < 0) {
+		perror("E: mmap'ing nbdtab");
+		exit(EXIT_FAILURE);
+	}
+
+	data = mmap(NULL, (size_t)size, PROT_READ, MAP_SHARED, fd, 0);
+	char *loc = strstr((const char*)data, cfgname);
+	if(!loc) {
+		goto out;
+	}
+	size_t l = strlen(cfgname) + 6;
+	*dev_ptr = malloc(l);
+	snprintf(*dev_ptr, l, "/dev/%s", cfgname);
+
+	size_t line_len, field_len, ws_len;
+#define CHECK_LEN field_len = strcspn(loc, fsep); ws_len = strspn(loc+field_len, fsep); if(field_len > line_len || line_len <= 0) { goto out; }
+#define MOVE_NEXT line_len -= field_len + ws_len; loc += field_len + ws_len
+	// find length of line
+	line_len = strcspn(loc, lsep);
+	// first field is the device node name, which we already know, so skip it
+	CHECK_LEN;
+	MOVE_NEXT;
+	// next field is the hostname
+	CHECK_LEN;
+	*hostn_ptr = strndup(loc, field_len);
+	MOVE_NEXT;
+	// third field is the export name
+	CHECK_LEN;
+	*name_ptr = strndup(loc, field_len);
+	if(ws_len + field_len > line_len) {
+		// optional last field is not there, so return success
+		retval = true;
+		goto out;
+	}
+	MOVE_NEXT;
+	CHECK_LEN;
+#undef CHECK_LEN
+#undef MOVE_NEXT
+	// fourth field is the options field, a comma-separated field of options
+	do {
+		if(!strncmp(loc, "bs=", 3)) {
+			*bs = (int)strtol(loc+3, &loc, 0);
+			goto next;
+		}
+		if(!strncmp(loc, "timeout=", 8)) {
+			*timeout = (int)strtol(loc+8, &loc, 0);
+			goto next;
+		}
+		if(!strncmp(loc, "port=", 5)) {
+			*port = strndup(loc+5, strcspn(loc+5, ","));
+			goto next;
+		}
+		if(!strncmp(loc, "persist", 7)) {
+			loc += 7;
+			*persist = 1;
+			goto next;
+		}
+		if(!strncmp(loc, "swap", 4)) {
+			*swap = 1;
+			loc += 4;
+			goto next;
+		}
+		if(!strncmp(loc, "sdp", 3)) {
+			*sdp = 1;
+			loc += 3;
+			goto next;
+		}
+		if(!strncmp(loc, "unix", 4)) {
+			*b_unix = 1;
+			loc += 4;
+			goto next;
+		}
+		// skip unknown options, with a warning unless they start with a '_'
+		l = strcspn(loc, ",");
+		if(*loc != '_') {
+			char* s = strndup(loc, l);
+			fprintf(stderr, "Warning: unknown option '%s' found in nbdtab file", s);
+			free(s);
+		}
+		loc += l;
+next:
+		if(*loc == ',') {
+			loc++;
+		}
+	} while(strcspn(loc, lsep) > 0);
+	retval = true;
+out:
+	if(data != NULL) {
+		munmap(data, size);
+	}
+	if(fd >= 0) {
+		close(fd);
+	}
+	return retval;
+}
+
 void setsizes(int nbd, u64 size64, int blocksize, u32 flags) {
 	unsigned long size;
 	int read_only = (flags & NBD_FLAG_READ_ONLY) ? 1 : 0;
@@ -368,8 +490,10 @@ void finish_sock(int sock, int nbd, int swap) {
 	if (ioctl(nbd, NBD_SET_SOCK, sock) < 0)
 		err("Ioctl NBD_SET_SOCK failed: %m\n");
 
+#ifndef __ANDROID__
 	if (swap)
 		mlockall(MCL_CURRENT | MCL_FUTURE);
+#endif
 }
 
 static int
@@ -398,8 +522,9 @@ void usage(char* errmsg, ...) {
 	} else {
 		fprintf(stderr, "nbd-client version %s\n", PACKAGE_VERSION);
 	}
-	fprintf(stderr, "Usage: nbd-client host port nbd_device [-block-size|-b block size] [-timeout|-t timeout] [-swap|-s] [-sdp|-S] [-persist|-p] [-nofork|-n] [-systemd-mark|-m]\n");
-	fprintf(stderr, "Or   : nbd-client -name|-N name host [port] nbd_device [-block-size|-b block size] [-timeout|-t timeout] [-swap|-s] [-sdp|-S] [-persist|-p] [-nofork|-n]\n");
+	fprintf(stderr, "Usage: nbd-client -name|-N name host [port] nbd_device\n\t[-block-size|-b block size] [-timeout|-t timeout] [-swap|-s] [-sdp|-S]\n\t[-persist|-p] [-nofork|-n] [-systemd-mark|-m]\n");
+	fprintf(stderr, "Or   : nbd-client -u (with same arguments as above)\n");
+	fprintf(stderr, "Or   : nbd-client nbdX\n");
 	fprintf(stderr, "Or   : nbd-client -d nbd_device\n");
 	fprintf(stderr, "Or   : nbd-client -c nbd_device\n");
 	fprintf(stderr, "Or   : nbd-client -h|--help\n");
@@ -408,7 +533,7 @@ void usage(char* errmsg, ...) {
 	fprintf(stderr, "Allowed values for blocksize are 512,1024,2048,4096\n"); /* will be checked in kernel :) */
 	fprintf(stderr, "Note, that kernel 2.4.2 and older ones do not work correctly with\n");
 	fprintf(stderr, "blocksizes other than 1024 without patches\n");
-	fprintf(stderr, "Default value for port with -N is 10809. Note that port must always be numeric\n");
+	fprintf(stderr, "Default value for port is 10809. Note that port must always be numeric\n");
 }
 
 void disconnect(char* device) {
@@ -426,7 +551,7 @@ void disconnect(char* device) {
 }
 
 int main(int argc, char *argv[]) {
-	char* port=NULL;
+	char* port=NBD_DEFAULT_PORT;
 	int sock, nbd;
 	int blocksize=1024;
 	char *hostname=NULL;
@@ -437,12 +562,13 @@ int main(int argc, char *argv[]) {
 	int sdp=0;
 	int G_GNUC_UNUSED nofork=0; // if -dNOFORK
 	u64 size64;
-	u32 flags;
+	uint16_t flags = 0;
 	int c;
 	int nonspecial=0;
-	char* name=NULL;
-	uint32_t needed_flags=0;
-	uint32_t cflags=0;
+	int b_unix=0;
+	char* name="";
+	uint16_t needed_flags=0;
+	uint32_t cflags=NBD_FLAG_C_FIXED_NEWSTYLE;
 	uint32_t opts=0;
 	sigset_t block, old;
 	struct sigaction sa;
@@ -459,12 +585,13 @@ int main(int argc, char *argv[]) {
 		{ "swap", no_argument, NULL, 's' },
 		{ "systemd-mark", no_argument, NULL, 'm' },
 		{ "timeout", required_argument, NULL, 't' },
+		{ "unix", no_argument, NULL, 'u' },
 		{ 0, 0, 0, 0 }, 
 	};
 
-	logging();
+	logging(MY_NAME);
 
-	while((c=getopt_long_only(argc, argv, "-b:c:d:hlnN:pSst:", long_options, NULL))>=0) {
+	while((c=getopt_long_only(argc, argv, "-b:c:d:hlnN:pSst:u", long_options, NULL))>=0) {
 		switch(c) {
 		case 1:
 			// non-option argument
@@ -491,7 +618,7 @@ int main(int argc, char *argv[]) {
 				case 1:
 					// port
 					if(!strtol(optarg, NULL, 0)) {
-						// not parseable as a number, assume it's the device and we have a name
+						// not parseable as a number, assume it's the device
 						nbddev = optarg;
 						nonspecial++;
 					} else {
@@ -521,11 +648,8 @@ int main(int argc, char *argv[]) {
 			exit(EXIT_SUCCESS);
 		case 'l':
 			needed_flags |= NBD_FLAG_FIXED_NEWSTYLE;
-			cflags |= NBD_FLAG_C_FIXED_NEWSTYLE;
 			opts |= NBDC_DO_LIST;
-			name="";
 			nbddev="";
-			port = NBD_DEFAULT_PORT;
 			break;
 		case 'm':
 			argv[0][0] = '@';
@@ -535,9 +659,6 @@ int main(int argc, char *argv[]) {
 			break;
 		case 'N':
 			name=optarg;
-			if(!port) {
-				port = NBD_DEFAULT_PORT;
-			}
 			break;
 		case 'p':
 			cont=1;
@@ -552,18 +673,44 @@ int main(int argc, char *argv[]) {
 		      timeout:
 			timeout=strtol(optarg, NULL, 0);
 			break;
+		case 'u':
+			b_unix = 1;
+			break;
 		default:
 			fprintf(stderr, "E: option eaten by 42 mice\n");
 			exit(EXIT_FAILURE);
 		}
 	}
 
-	if((!port && !name) || !hostname || !nbddev) {
-		usage("not enough information specified");
+#ifdef __ANDROID__
+  if (swap)
+    err("swap option unsupported on Android because mlockall is unsupported.");
+#endif
+	if(hostname) {
+		if((!name || !nbddev) && !(opts & NBDC_DO_LIST)) {
+			if(!strncmp(hostname, "nbd", 3) || !strncmp(hostname, "/dev/nbd", 8)) {
+				if(!get_from_config(hostname, &name, &nbddev, &hostname, &blocksize, &timeout, &cont, &swap, &sdp, &b_unix, &port)) {
+					usage("no valid configuration for specified device found", hostname);
+					exit(EXIT_FAILURE);
+				}
+			} else {
+				usage("not enough information specified, and argument didn't look like an nbd device");
+				exit(EXIT_FAILURE);
+			}
+		}
+	} else {
+		usage("no information specified");
 		exit(EXIT_FAILURE);
 	}
 
-	sock = opennet(hostname, port, sdp);
+	if(strlen(name)==0 && !(opts & NBDC_DO_LIST)) {
+		printf("Warning: the oldstyle protocol is no longer supported.\nThis method now uses the newstyle protocol with a default export\n");
+	}
+
+	if (b_unix)
+		sock = openunix(hostname);
+	else
+		sock = opennet(hostname, port, sdp);
 	if (sock < 0)
 		exit(EXIT_FAILURE);
 
@@ -617,8 +764,12 @@ int main(int argc, char *argv[]) {
 			 * does not return until the NBD device has
 			 * disconnected.
 			 */
+			struct timespec req = {
+				.tv_sec = 0,
+				.tv_nsec = 100000000,
+			};
 			while(check_conn(nbddev, 0)) {
-				sleep(1);
+				nanosleep(&req, NULL);
 			}
 			open(nbddev, O_RDONLY);
 			exit(0);
@@ -635,12 +786,15 @@ int main(int argc, char *argv[]) {
 			} else {
 				if(cont) {
 					u64 new_size;
-					u32 new_flags;
+					uint16_t new_flags;
 
 					close(sock); close(nbd);
 					for (;;) {
 						fprintf(stderr, " Reconnecting\n");
-						sock = opennet(hostname, port, sdp);
+						if (b_unix)
+							sock = openunix(hostname);
+						else
+							sock = opennet(hostname, port, sdp);
 						if (sock >= 0)
 							break;
 						sleep (1);
